// copyright me
//

window.lastLoaderErrors = [];
window.lastWindowErrors = [];

window.onerror = function(msg, url, line, col, error) {
  // Note that col & error are new to the HTML 5 spec and may not be
  // supported in every browser.  It worked for me in Chrome.
  var extra = !col ? '' : '\ncolumn: ' + col;
  extra += !error ? '' : '\nerror: ' + error;

  var errorString = msg + "\nurl: " + url + "\nline: " + line + extra;

  console.log(msg, url, line, col, error);
  window.lastError = {msg, url, line, col, error};

  var userInfo;
  var userId;
  try {
    userInfo = (window.localStorage && window.localStorage.getItem("loggedInUser"));
    try {
      userId = userInfo && JSON.parse(userInfo).id;
    } catch (err) {
      // nothin'
    }
  } catch (err) {
    // do nothing, just trying to log
  }

  window.lastWindowErrors.push({
    category: 'error',
    logName: 'clientError',
    data: {
      errorString: errorString,
      userId: userId,
      userInfo: userInfo,
    }
  });
};

try {
	var IS_PRODUCTION = window.IS_PRODUCTION;
	var LATEST_BUNDLE_KEY = "LATEST_BUNDLE";
	var LATEST_BUNDLE_VERSION_KEY = "LATEST_BUNDLE_VERSION";
	var IS_USING_COMPRESSED_APP_KEY = "IS_USING_COMPRESSED_APP";
	var DEFAULT_BUNDLE_PATH = "bundle.js";
	var SCRIPT_LOOKUP_DELAY_INITIAL = IS_PRODUCTION
		?	1000 * 5
		: 1000 * 5;
	var SCRIPT_LOOKUP_DELAY_RECURRING = IS_PRODUCTION
		? 1000 * 60 * 60
		: 1000 * 60 * 10;
	var MIN_REFETCH_TIME = IS_PRODUCTION
		? 1000 * 60 * 10
		: 1000 * 60 * 10;

	var lastFetch;
	var cachedVersion = parseInt(localStorage.getItem(LATEST_BUNDLE_VERSION_KEY)) || 0;

	loadLatestScript();
	setTimeout(function() {
		checkForNewestScript();
	}, SCRIPT_LOOKUP_DELAY_INITIAL);

	setInterval(function() {
		checkForNewestScript();
	}, SCRIPT_LOOKUP_DELAY_RECURRING);

	window.checkForUpdate = function(restartOnUpdate) {
		checkForNewestScript(restartOnUpdate);
	}

	window.reloadCordova = function(restartOnUpdate) {
		window.location = 'index.html';
	}

	window.applyUpdate = function() {
		// i think this works because the router changes the base route to like ://file
		// so this re-points it to 'index.html'
		window.reloadCordova();
	}

	var basePath = IS_PRODUCTION
		? 'https://www.guilded.gg'
		// : 'http://localhost:8005';
		: 'http://tarobi-dev-test.com';

	function checkForNewestScript(restartOnUpdate) {
		if (lastFetch && (new Date() - lastFetch > MIN_REFETCH_TIME)) {
			return;
		}

		var xhr = new XMLHttpRequest();
		xhr.open( 'GET', basePath + '/appInfo.json'  + '?' + Math.random().toString(36).substring(7) );
		xhr.onreadystatechange = function() {
			if ( xhr.readyState === 4 ) {
				if ( ( xhr.status === 200 ) ||
						( ( xhr.status === 0 ) && xhr.responseText ) ) {
					var appInfo = JSON.parse(xhr.responseText);

					if (appInfo.appVersion > cachedVersion) {
						localStorage.setItem(IS_USING_COMPRESSED_APP_KEY, true);
						checkAndStoreScript(
							appInfo.appVersion,
							window.IS_NATIVE ? appInfo.compressedAppPath : appInfo.compressedElectronPath
						);

						if (restartOnUpdate) {
							window.reloadCordova();
						}
					}
				} else {
				}
			}
		};

		// By default XHRs never timeout, and even Chrome doesn't implement the
		// spec for xhr.timeout. So we do it ourselves.
		setTimeout( function () {
			if( xhr.readyState < 4 ) {
				xhr.abort();
			}
		}, 10000);

		xhr.send();
	}

	function checkAndStoreScript(version, path) {
		var xhr2 = new XMLHttpRequest();
		var scriptPath = basePath + '/' + path;
		xhr2.open('GET', scriptPath);
		xhr2.onreadystatechange = function() {
			if ( xhr2.readyState === 4 ) {
				if ( ( xhr2.status === 200 ) ||
						( ( xhr2.status === 0 ) && xhr2.responseText ) ) {
					if (!xhr2.responseText) {
						if (!IS_PRODUCTION) {
							alert("Didn't get response back from bundle path. This is bad. Abort.");
						}
						return;
					}
					try {
						localStorage.setItem(LATEST_BUNDLE_KEY, xhr2.responseText);
						localStorage.setItem(LATEST_BUNDLE_VERSION_KEY, version);
						if (!IS_PRODUCTION) {
							alert(version);
							window.downloadedAppUpdateThisSession = true;
						}
						if (window.logSuccessfulUpdate) {
							window.logSuccessfulUpdate(version);
						}
						lastFetch = new Date();
					} catch (err) {
						window.lastLoaderErrors.push(err);
						if (window.logFailedUpdate) {
							window.logFailedUpdate(window.lastLoaderErrors[window.lastLoaderErrors.length - 1]);
						}
					}
				} else {
					window.lastLoaderErrors.push("weird response " + xhr2.status + " " + xhr2.responseText);
					if (window.logFailedUpdate) {
						window.logFailedUpdate(window.lastLoaderErrors[window.lastLoaderErrors.length - 1]);
					}
				}
			}
		}

		// By default XHR2s never timeout, and even Chrome doesn't implement the
		// spec for xhr2.timeout. So we do it ourselves.
		setTimeout( function () {
			if( xhr2.readyState < 4 ) {
				xhr2.abort();
			}
		}, 10000 );

		xhr2.send();
	}

	function loadLatestScript() {
		var cachedScript = localStorage.getItem(LATEST_BUNDLE_KEY);

		if (!cachedScript) {
			loadDefaultScript();
			return;
		}

		var head = document.head || document.getElementsByTagName('head')[0];
		var script = document.createElement('script');

		if (localStorage.getItem(IS_USING_COMPRESSED_APP_KEY)) {
			var now = performance.now();
			cachedScript = window.LZString.decompressFromEncodedURIComponent(cachedScript);
			window.decompressTime = performance.now() - now;

			if (!cachedScript) {
				loadDefaultScript();
				return;
			}
		}
		try {
			script.defer = true;
			script.text = cachedScript;
			head.appendChild(script);
			window.loadedCompressedScript = true;
			window.loadedCompressedScriptVersionNum = localStorage.getItem(LATEST_BUNDLE_VERSION_KEY);
			return;
		} catch (err) {
			window.lastLoaderErrors.push(err);
			if (window.logFailedUpdate) {
				window.logFailedUpdate(window.lastLoaderErrors[window.lastLoaderErrors.length - 1]);
			}
		}

		loadDefaultScript();
	}

	function loadDefaultScript() {
		window.loadedDefaultScript = true;
		var head = document.head || document.getElementsByTagName('head')[0];
		var script = document.createElement('script');
		script.defer = true;
		script.src = DEFAULT_BUNDLE_PATH;
		head.appendChild(script);
	}
} catch (err) {
	window.lastLoaderErrors.push(err);
}