const electron = require('electron')
// Module to control application life.
const app = electron.app;
// Module to create native browser window.
const BrowserWindow = electron.BrowserWindow;
const Menu = electron.Menu;

const ipc = electron.ipcMain;
ipc.on('fb-login', function (event) {
  loginWithFBElectron(event);
});

ipc.on('steam-login', function (event, steamUrl) {
  loginWithSteamElectron(steamUrl);
});

ipc.on('discord-login', function (event, discordUrl) {
  loginWithDiscordElectron(discordUrl);
});

const shell = electron.shell;
// hacky - our first nav is just on signin, so we don't want to open browser window
// let hasEatenFirstNav = false;
ipc.on('url-change', function (event, location) {
  // if (!hasEatenFirstNav) {
  //   hasEatenFirstNav = true;
  //   return;
  // }

  if ((location.hash || '')[0] === '/') {
    location.hash = location.hash.slice(1);
  }

  // incase there are two slashes lol copypasta shitcode
  if ((location.hash || '')[0] === '/') {
    location.hash = location.hash.slice(1);
  }

  if (
    location.hash
    && location.hash !== '/'
    && location.hash !== 'teams'
  ) {
    shell.openExternal('https://www.guilded.gg/' + location.hash);
  }
});

ipc.on('notif-count', function(event, count) {
  setBadge(count || '');
});


// Keep a global reference of the window object, if you don't, the window will
// be closed automatically when the JavaScript object is garbage collected.
let mainWindow;
function createWindow () {
  var menu = defaultMenu()

  // Add custom menu
  // menu.splice(4, 0, {
  //   label: 'Custom',
  //   submenu: [
  //     {
  //       label: 'Do something',
  //       click: function(item, focusedWindow) {
  //         dialog.showMessageBox({message: 'Do something', buttons: ['OK'] })
  //       }
  //     }
  //   ]
  // })

  // Set top-level application menu, using modified template
  Menu.setApplicationMenu(Menu.buildFromTemplate(menu));

  // Create the browser window.
  mainWindow = new BrowserWindow({
    width: 950,
    height: 690,
    titleBarStyle: 'hidden-inset',
    backgroundColor: '#212124',
    minHeight: 690,
    minWidth: 950,
    title: "Guilded Chat",
    autoHideMenuBar: true,
    darkTheme: true,
    icon: 'Guilded.ico',
    transparent: true,
    frame: false,
    webPreferences: {
      webSecurity: false,
      plugins: true,
      backgroundThrottling: false,
    }
  });

  // and load the index.html of the app.
  mainWindow.loadURL(`file://${__dirname}/index.html`)

  // Open the DevTools.
  // mainWindow.webContents.openDevTools();

  // Emitted when the window is closed.
  mainWindow.on('closed', function () {
    // Dereference the window object, usually you would store windows
    // in an array if your app supports multi windows, this is the time
    // when you should delete the corresponding element.
    mainWindow = null
  });

  // mainWindow.loadURL('http://tarobi-dev-test.com');

  // window.loginWithFBElectron = loginWithFBElectron;
}

// This method will be called when Electron has finished
// initialization and is ready to create browser windows.
// Some APIs can only be used after this event occurs.
app.on('ready', createWindow)

// Quit when all windows are closed.
app.on('window-all-closed', function () {
  // On OS X it is common for applications and their menu bar
  // to stay active until the user quits explicitly with Cmd + Q
  if (process.platform !== 'darwin') {
    app.quit()
  }
})

app.on('activate', function () {
  // On OS X it's common to re-create a window in the app when the
  // dock icon is clicked and there are no other windows open.
  if (mainWindow === null) {
    createWindow()
  }
})

// In this file you can include the rest of your app's specific main process
// code. You can also put them in separate files and require them here.

function loginWithFBElectron(event) {
  // Build the OAuth consent page URL
  var authWindow = new BrowserWindow({
    width: 800,
    height: 600,
    show: false,
    // titleBarStyle: 'hidden-inset',
    webPreferences: {
      webSecurity: false,
      plugins: true,
      backgroundThrottling: false,
      nodeIntegration: false,
    }
  });
  var facebookUrl = 'https://www.facebook.com/dialog/oauth?client_id=120226285050834&redirect_uri=https://www.facebook.com/connect/login_success.html';
  authWindow.loadURL(facebookUrl);
  authWindow.show();

  function handleCallback (url) {
    var raw_code = /code=([^&]*)/.exec(url) || null;
    var code = (raw_code && raw_code.length > 1) ? raw_code[1] : null;
    var error = /\?error=(.+)$/.exec(url);

    if (code || error) {
      // Close the browser if code found or error
      authWindow.destroy();
    }

    // If there is a code, proceed to get token from github
    if (code) {
      event.sender.send('fb-login-success', code);
      // onSuccess(code);
    } else if (error) {
      event.sender.send('fb-login-error', error);
    }
  }

  // Handle the response from GitHub - See Update from 4/12/2015

  authWindow.webContents.on('will-navigate', function (event, url) {
    handleCallback(url);
  });

  authWindow.webContents.on('did-get-redirect-request', function (event, oldUrl, newUrl) {
    handleCallback(newUrl);
  });

  // Reset the authWindow on close
  authWindow.on('close', function() {
      authWindow = null;
  }, false);
}

setBadge = function (text) {
  if (process.platform === "darwin") {
    app.dock.setBadge("" + text);
  } else if (process.platform === "win32") {
    if (text === "") {
      mainWindow.setOverlayIcon(null, "");
    } else {
      mainWindow.setOverlayIcon(`${__dirname}/NotificationIcon.png`, "New message");
    }

    // // Create badge
    // var canvas = document.createElement("canvas");
    // canvas.height = 140;
    // canvas.width = 140;
    // var ctx = canvas.getContext("2d");
    // ctx.fillStyle = "red";
    // ctx.beginPath();
    // ctx.ellipse(70, 70, 70, 70, 0, 0, 2 * Math.PI);
    // ctx.fill();
    // ctx.textAlign = "center";
    // ctx.fillStyle = "white";

    // if (text.length > 2) {
    //   ctx.font = "75px sans-serif";
    //   ctx.fillText("" + text, 70, 98);
    // } else if (text.length > 1) {
    //   ctx.font = "100px sans-serif";
    //   ctx.fillText("" + text, 70, 105);
    // } else {
    //   ctx.font = "125px sans-serif";
    //   ctx.fillText("" + text, 70, 112);
    // }

    // var badgeDataURL = canvas.toDataURL();
    // var img = NativeImage.createFromDataUrl(badgeDataURL);

    // mainWindow.setOverlayIcon(img, text);
  }
};

function loginWithSteamElectron(steamUrl, onSuccess, onFail) {
  var authWindow = new BrowserWindow({
    width: 800,
    height: 600,
    show: false,
    // titleBarStyle: 'hidden-inset',
    webPreferences: {
      webSecurity: false,
      plugins: true,
      backgroundThrottling: false,
      nodeIntegration: false,
    }
  });
  authWindow.loadURL(steamUrl);
  authWindow.show();

  ipc.on('steam-login-complete', function (event, steamUrl) {
    authWindow.destroy();
  });

  // Reset the authWindow on close
  authWindow.on('close', function() {
      authWindow = null;
  }, false);
}

function loginWithDiscordElectron(discordUrl, onSuccess, onFail) {
  var authWindow = new BrowserWindow({
    width: 800,
    height: 830,
    show: false,
    // titleBarStyle: 'hidden-inset',
    webPreferences: {
      webSecurity: false,
      plugins: true,
      backgroundThrottling: false,
      nodeIntegration: false,
    }
  });
  authWindow.loadURL(discordUrl);
  authWindow.show();

  ipc.on('discord-login-complete', function (event, discordUrl) {
    authWindow.destroy();
  });

  // Reset the authWindow on close
  authWindow.on('close', function() {
      authWindow = null;
  }, false);
}


function defaultMenu() {
  var template = [
    {
      label: 'Edit',
      submenu: [
        {
          label: 'Undo',
          accelerator: 'CmdOrCtrl+Z',
          role: 'undo'
        },
        {
          label: 'Redo',
          accelerator: 'Shift+CmdOrCtrl+Z',
          role: 'redo'
        },
        {
          type: 'separator'
        },
        {
          label: 'Cut',
          accelerator: 'CmdOrCtrl+X',
          role: 'cut'
        },
        {
          label: 'Copy',
          accelerator: 'CmdOrCtrl+C',
          role: 'copy'
        },
        {
          label: 'Paste',
          accelerator: 'CmdOrCtrl+V',
          role: 'paste'
        },
        {
          label: 'Select All',
          accelerator: 'CmdOrCtrl+A',
          role: 'selectall'
        },
      ]
    },
    {
      label: 'View',
      submenu: [
        {
          label: 'Reload',
          accelerator: 'CmdOrCtrl+R',
          click: function(item, focusedWindow) {
            if (focusedWindow)
              focusedWindow.reload();
          }
        },
        {
          label: 'Toggle Full Screen',
          accelerator: (function() {
            if (process.platform === 'darwin')
              return 'Ctrl+Command+F';
            else
              return 'F11';
          })(),
          click: function(item, focusedWindow) {
            if (focusedWindow)
              focusedWindow.setFullScreen(!focusedWindow.isFullScreen());
          }
        },
        {
          label: 'Toggle Developer Tools',
          accelerator: (function() {
            if (process.platform === 'darwin')
              return 'Alt+Command+I';
            else
              return 'Ctrl+Shift+I';
          })(),
          click: function(item, focusedWindow) {
            if (focusedWindow)
              focusedWindow.toggleDevTools();
          }
        },
      ]
    },
    {
      label: 'Window',
      role: 'window',
      submenu: [
        {
          label: 'Minimize',
          accelerator: 'CmdOrCtrl+M',
          role: 'minimize'
        },
        {
          label: 'Close',
          accelerator: 'CmdOrCtrl+W',
          role: 'close'
        },
      ]
    },
    {
      label: 'Help',
      role: 'help',
      submenu: [
        {
          label: 'Learn More',
          click: function() { shell.openExternal('http://electron.atom.io') }
        },
      ]
    },
  ];

  if (process.platform === 'darwin') {
    var name = app.getName();
    template.unshift({
      label: name,
      submenu: [
        {
          label: 'About ' + name,
          role: 'about'
        },
        {
          type: 'separator'
        },
        {
          label: 'Services',
          role: 'services',
          submenu: []
        },
        {
          type: 'separator'
        },
        {
          label: 'Hide ' + name,
          accelerator: 'Command+H',
          role: 'hide'
        },
        {
          label: 'Hide Others',
          accelerator: 'Command+Shift+H',
          role: 'hideothers'
        },
        {
          label: 'Show All',
          role: 'unhide'
        },
        {
          type: 'separator'
        },
        {
          label: 'Quit',
          accelerator: 'Command+Q',
          click: function() { app.quit(); }
        },
      ]
    });
    var windowMenu = template.find(function(m) { return m.role === 'window' })
    if (windowMenu) {
      windowMenu.submenu.push(
        {
          type: 'separator'
        },
        {
          label: 'Bring All to Front',
          role: 'front'
        }
      );
    }
  }

  return template;
}